@@ -90,9 +90,9 @@ mod or_chip {
         pub fn new(state: &mut dyn State) -> Chip {
             let chip = Chip::new(state, |chip| (Box::new(Self::new_priv()) as _, chip));
             let binding = Binding2::new(state, (), |(), in_1, in_2| Some(in_1 | in_2));
+            OrLegs::OUT.bind(state, chip.legs(), binding);
             binding.set_source_1(state, &mut OrLegs::IN_1.value_source(chip.legs()));
             binding.set_source_2(state, &mut OrLegs::IN_2.value_source(chip.legs()));
-            OrLegs::OUT.bind(state, chip.legs(), binding);
             chip
         }
     }
@@ -118,8 +118,8 @@ mod not_chip {
         pub fn new(state: &mut dyn State) -> Chip {
             let chip = Chip::new(state, |chip| (Box::new(Self::new_priv()) as _, chip));
             let binding = Binding1::new(state, (), |(), in_1: bool| Some(!in_1));
-            binding.set_source_1(state, &mut NotLegs::IN_.value_source(chip.legs()));
             NotLegs::OUT.bind(state, chip.legs(), binding);
+            binding.set_source_1(state, &mut NotLegs::IN_.value_source(chip.legs()));
             chip
         }
     }
@@ -128,11 +128,13 @@ mod not_chip {
 }
 
 use circuit::*;
-use dep_obj::binding::{Binding1, Bindings, EventBinding0};
-use dyn_context::state::{State, StateRefMut};
+use dep_obj::binding::{Binding1, Bindings, b_immediate};
+use dyn_context::state::{State, StateExt, StateRefMut};
 use not_chip::*;
 use or_chip::*;
 use std::any::{Any, TypeId};
+use std::fmt::Write;
+use dep_obj::Change;
 
 #[derive(Debug, Clone)]
 struct TriggerChips {
@@ -147,11 +149,14 @@ struct TriggerState {
     bindings: Bindings,
     circuit: Circuit,
     chips: TriggerChips,
+    log: String,
 }
 
 impl State for TriggerState {
     fn get_raw(&self, ty: TypeId) -> Option<&dyn Any> {
-        if ty == TypeId::of::<Bindings>() {
+        if ty == TypeId::of::<TriggerState>() {
+            Some(self)
+        } else if ty == TypeId::of::<Bindings>() {
             Some(&self.bindings)
         } else if ty == TypeId::of::<Circuit>() {
             Some(&self.circuit)
@@ -163,7 +168,9 @@ impl State for TriggerState {
     }
 
     fn get_mut_raw(&mut self, ty: TypeId) -> Option<&mut dyn Any> {
-        if ty == TypeId::of::<Bindings>() {
+        if ty == TypeId::of::<TriggerState>() {
+            Some(self)
+        } else if ty == TypeId::of::<Bindings>() {
             Some(&mut self.bindings)
         } else if ty == TypeId::of::<Circuit>() {
             Some(&mut self.circuit)
@@ -187,40 +194,49 @@ fn main() {
         circuit,
         bindings,
         chips: chips.clone(),
+        log: String::new(),
     };
 
     let not_1_out_to_or_2_in = Binding1::new(state, (), |(), value| Some(value));
-    not_1_out_to_or_2_in.set_source_1(state, &mut NotLegs::OUT.value_source(chips.not_1.legs()));
     OrLegs::IN_2.bind(state, chips.or_2.legs(), not_1_out_to_or_2_in);
+    not_1_out_to_or_2_in.set_source_1(state, &mut NotLegs::OUT.value_source(chips.not_1.legs()));
     let not_2_out_to_or_1_in = Binding1::new(state, (), |(), value| Some(value));
-    not_2_out_to_or_1_in.set_source_1(state, &mut NotLegs::OUT.value_source(chips.not_2.legs()));
     OrLegs::IN_2.bind(state, chips.or_1.legs(), not_2_out_to_or_1_in);
+    not_2_out_to_or_1_in.set_source_1(state, &mut NotLegs::OUT.value_source(chips.not_2.legs()));
     let or_1_out_to_not_1_in = Binding1::new(state, (), |(), value| Some(value));
-    or_1_out_to_not_1_in.set_source_1(state, &mut OrLegs::OUT.value_source(chips.or_1.legs()));
     NotLegs::IN_.bind(state, chips.not_1.legs(), or_1_out_to_not_1_in);
+    or_1_out_to_not_1_in.set_source_1(state, &mut OrLegs::OUT.value_source(chips.or_1.legs()));
     let or_2_out_to_not_2_in = Binding1::new(state, (), |(), value| Some(value));
-    or_2_out_to_not_2_in.set_source_1(state, &mut OrLegs::OUT.value_source(chips.or_2.legs()));
     NotLegs::IN_.bind(state, chips.not_2.legs(), or_2_out_to_not_2_in);
+    or_2_out_to_not_2_in.set_source_1(state, &mut OrLegs::OUT.value_source(chips.or_2.legs()));
 
-    let print_out = EventBinding0::new(state, (), |_, (), x| x.cloned());
-    print_out.set_event_source(state, &mut NotLegs::OUT.change_source(chips.not_2.legs()));
-    print_out.set_target_fn(state, (), |_, (), change| {
-        let old = if *change.old() { "1" } else { "0" };
-        let new = if *change.new() { "1" } else { "0" };
-        println!("{} -> {}", old, new);
+    let print_out = Binding1::new(state, (), |(), change: Option<Change<bool>>| change);
+    print_out.set_target_fn(state, (), |state, (), change| {
+        let state: &mut TriggerState = state.get_mut();
+        let old = if change.old { "1" } else { "0" };
+        let new = if change.new { "1" } else { "0" };
+        writeln!(state.log, "{} -> {}", old, new).unwrap();
     });
-    OrLegs::IN_1.set(state, chips.or_1.legs(), true);
-    OrLegs::IN_1.set(state, chips.or_1.legs(), false);
-    OrLegs::IN_1.set(state, chips.or_2.legs(), true);
-    OrLegs::IN_1.set(state, chips.or_2.legs(), false);
-    OrLegs::IN_1.set(state, chips.or_1.legs(), true);
-    OrLegs::IN_1.set(state, chips.or_1.legs(), false);
-    OrLegs::IN_1.set(state, chips.or_2.legs(), true);
-    OrLegs::IN_1.set(state, chips.or_2.legs(), false);
+    print_out.set_source_1(state, &mut NotLegs::OUT.change_source(chips.not_2.legs()));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_1.legs(), true));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_1.legs(), false));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_2.legs(), true));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_2.legs(), false));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_1.legs(), true));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_1.legs(), false));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_2.legs(), true));
+    b_immediate(OrLegs::IN_1.set(state, chips.or_2.legs(), false));
 
     print_out.drop_binding(state);
     chips.or_1.drop_chip(state);
     chips.or_2.drop_chip(state);
     chips.not_2.drop_chip(state);
     chips.not_1.drop_chip(state);
+
+    print!("{}", state.log);
+    assert_eq!(state.log, "\
+        1 -> 0\n\
+        0 -> 1\n\
+        1 -> 0\n\
+    ");
 }
